#! /usr/bin/env python
# -*- coding:utf-8 -*-
"""
@author  : MG
@Time    : 2018/10/25 17:42
@File    : other_tables.py
@contact : mmmaaaggg@163.com
@desc    : 
"""
from ibats_bitmex_feeder.backend.orm import dynamic_load_table_model, table_model_dic


if len(table_model_dic) == 0:
    dynamic_load_table_model()

INSTRUMENT_INFO_TABLE_NAME = 'bitmex_instrument'
# 类型：type(instrument_info_table) :  <class 'sqlalchemy.sql.schema.Table'>
# 用法：session.execute(instrument_info_table.count()).scalar()
instrument_info_table = table_model_dic[INSTRUMENT_INFO_TABLE_NAME]



#! /usr/bin/env python
# -*- coding:utf-8 -*-
"""
@author  : MG
@Time    : 2018/9/27 14:58
@File    : mess.py
@contact : mmmaaaggg@163.com
@desc    : 
"""
import pandas as pd
from bravado.client import CallableOperation
from ibats_common.utils.mess import try_n_times, log_param_when_exception
import logging
from bravado.requests_client import RequestsResponseAdapter
from bravado.exception import HTTPForbidden

logger = logging.getLogger()


# @log_param_when_exception
@try_n_times(3, sleep_time=1, logger=logger, exception_exclusion_set={HTTPForbidden})
def try_call_func(func: CallableOperation, *args, **kwargs) -> (list, RequestsResponseAdapter):
    """
    请求频率限制:对我们的 REST API 的请求频率限于每5分钟300次。 此计数连续补充。 如果你没有登录，你的请求是每5分钟150次。 https://www.bitmex.com/app/restAPI
    :param func:
    :param args:
    :param kwargs:
    :return:
    """
    return func(*args, **kwargs).result()


def load_list_against_pagination(func: CallableOperation, page_no_since=0, count=500,
                               page_no_max=None, **kwargs) -> list:
    """
    调用接口函数，自动翻译加载全部数据并返回结果
    :param func:
    :param page_no_since:
    :param count:
    :param page_no_max:
    :param drop_duplicates:
    :return:
    """
    page_no = page_no_since
    ret_list = []
    param_str = ', '.join(
        ['{key}={value}'.format(key=str(key), value=str(value))
         for key, value in kwargs.items()]
    )
    while True:
        logger.debug('%s call %s(start=%s, count=%s, %s)',
                     func.operation.path_name, func.operation.operation_id, page_no, count, param_str)
        try:
            data_list, rsp = try_call_func(func, start=page_no, count=count, **kwargs)
        except:
            break
        if rsp is None:
            break
        if rsp.status_code != 200:
            logger.error("%s(start=%d, count=%d, %s) error status_code=%d, reason=''",
                         func.operation.path_name, page_no, count, param_str, rsp.status_code, rsp.reason)
        if data_list is None or len(data_list) == 0:
            break
        ret_list.extend(data_list)
        page_no += count
        if page_no_max is not None and page_no > page_no_max:
            break
    return ret_list


def load_df_against_pagination(func: CallableOperation, page_no_since=0, count=500,
                               page_no_max=None, drop_duplicates=True, **kwargs) -> pd.DataFrame:
    """
    调用接口函数，自动翻译加载全部数据并返回结果
    :param func:
    :param page_no_since:
    :param count:
    :param page_no_max:
    :param drop_duplicates:
    :return:
    """
    ret_list = load_list_against_pagination(func, page_no_since, count, page_no_max, **kwargs)

    if len(ret_list) > 0:
        ret_df = pd.DataFrame(ret_list)
        if drop_duplicates:
            ret_df.drop_duplicates(inplace=True)
    else:
        ret_df = None
    return ret_df


if __name__ == "__main__":
    import bitmex
    from ibats_bitmex_feeder.config import config

    api = bitmex.bitmex(test=config.TEST_NET)
    func = api.Instrument.Instrument_get
    # 测试 try_call_func 接口
    data_list, rsp = try_call_func(func, start=0, count=10)
    print(len(data_list))

    # 测试 load_against_pagination 接口
    ret_list = load_df_against_pagination(func)
    print(len(ret_list))



#! /usr/bin/env python
# -*- coding:utf-8 -*-
"""
@author  : MG
@Time    : 2018/6/12 13:02
@File    : orm.py
@contact : mmmaaaggg@163.com
@desc    :
"""
from sqlalchemy import Column, Integer, String, UniqueConstraint, TIMESTAMP, MetaData, Table
from sqlalchemy.dialects.mysql import DOUBLE
from sqlalchemy.ext.declarative import declarative_base
from ibats_common.utils.db import with_db_session, get_db_session
from ibats_bitmex_feeder.backend import engine_md
from ibats_bitmex_feeder.config import config
import logging

logger = logging.getLogger()
BaseModel = declarative_base()
table_model_dic = {}


class MDMin1(BaseModel):
    __tablename__ = 'md_bitmext_min1'
    timestamp = Column(TIMESTAMP, primary_key=True)
    symbol = Column(String(10), primary_key=True)
    open = Column(DOUBLE)
    high = Column(DOUBLE)
    low = Column(DOUBLE)
    close = Column(DOUBLE)
    trades = Column(DOUBLE)
    volume = Column(DOUBLE)
    vwap = Column(DOUBLE)
    lastSize = Column(DOUBLE)
    turnover = Column(DOUBLE)
    homeNotional = Column(DOUBLE)
    foreignNotional = Column(DOUBLE)


class MDMin1Temp(BaseModel):
    __tablename__ = 'md_bitmext_min1_temp'
    timestamp = Column(TIMESTAMP, primary_key=True)
    symbol = Column(String(10), primary_key=True)
    open = Column(DOUBLE)
    high = Column(DOUBLE)
    low = Column(DOUBLE)
    close = Column(DOUBLE)
    trades = Column(DOUBLE)
    volume = Column(DOUBLE)
    vwap = Column(DOUBLE)
    lastSize = Column(DOUBLE)
    turnover = Column(DOUBLE)
    homeNotional = Column(DOUBLE)
    foreignNotional = Column(DOUBLE)


class MDMin5(BaseModel):
    __tablename__ = 'md_bitmext_min5'
    timestamp = Column(TIMESTAMP, primary_key=True)
    symbol = Column(String(10), primary_key=True)
    open = Column(DOUBLE)
    high = Column(DOUBLE)
    low = Column(DOUBLE)
    close = Column(DOUBLE)
    trades = Column(DOUBLE)
    volume = Column(DOUBLE)
    vwap = Column(DOUBLE)
    lastSize = Column(DOUBLE)
    turnover = Column(DOUBLE)
    homeNotional = Column(DOUBLE)
    foreignNotional = Column(DOUBLE)


class MDMin5Temp(BaseModel):
    __tablename__ = 'md_bitmext_min5_temp'
    timestamp = Column(TIMESTAMP, primary_key=True)
    symbol = Column(String(10), primary_key=True)
    open = Column(DOUBLE)
    high = Column(DOUBLE)
    low = Column(DOUBLE)
    close = Column(DOUBLE)
    trades = Column(DOUBLE)
    volume = Column(DOUBLE)
    vwap = Column(DOUBLE)
    lastSize = Column(DOUBLE)
    turnover = Column(DOUBLE)
    homeNotional = Column(DOUBLE)
    foreignNotional = Column(DOUBLE)


class MDHour1(BaseModel):
    __tablename__ = 'md_bitmext_hour1'
    timestamp = Column(TIMESTAMP, primary_key=True)
    symbol = Column(String(10), primary_key=True)
    open = Column(DOUBLE)
    high = Column(DOUBLE)
    low = Column(DOUBLE)
    close = Column(DOUBLE)
    trades = Column(DOUBLE)
    volume = Column(DOUBLE)
    vwap = Column(DOUBLE)
    lastSize = Column(DOUBLE)
    turnover = Column(DOUBLE)
    homeNotional = Column(DOUBLE)
    foreignNotional = Column(DOUBLE)


class MDHour1Temp(BaseModel):
    __tablename__ = 'md_bitmext_hour1_temp'
    timestamp = Column(TIMESTAMP, primary_key=True)
    symbol = Column(String(10), primary_key=True)
    open = Column(DOUBLE)
    high = Column(DOUBLE)
    low = Column(DOUBLE)
    close = Column(DOUBLE)
    trades = Column(DOUBLE)
    volume = Column(DOUBLE)
    vwap = Column(DOUBLE)
    lastSize = Column(DOUBLE)
    turnover = Column(DOUBLE)
    homeNotional = Column(DOUBLE)
    foreignNotional = Column(DOUBLE)


class MDDaily(BaseModel):
    __tablename__ = 'md_bitmext_daily'
    timestamp = Column(TIMESTAMP, primary_key=True)
    symbol = Column(String(10), primary_key=True)
    open = Column(DOUBLE)
    high = Column(DOUBLE)
    low = Column(DOUBLE)
    close = Column(DOUBLE)
    trades = Column(DOUBLE)
    volume = Column(DOUBLE)
    vwap = Column(DOUBLE)
    lastSize = Column(DOUBLE)
    turnover = Column(DOUBLE)
    homeNotional = Column(DOUBLE)
    foreignNotional = Column(DOUBLE)


class MDDailyTemp(BaseModel):
    __tablename__ = 'md_bitmext_daily_temp'
    timestamp = Column(TIMESTAMP, primary_key=True)
    symbol = Column(String(10), primary_key=True)
    open = Column(DOUBLE)
    high = Column(DOUBLE)
    low = Column(DOUBLE)
    close = Column(DOUBLE)
    trades = Column(DOUBLE)
    volume = Column(DOUBLE)
    vwap = Column(DOUBLE)
    lastSize = Column(DOUBLE)
    turnover = Column(DOUBLE)
    homeNotional = Column(DOUBLE)
    foreignNotional = Column(DOUBLE)


def init(alter_table=False):
    BaseModel.metadata.create_all(engine_md)
    if alter_table:
        with with_db_session(engine=engine_md) as session:
            for table_name, _ in BaseModel.metadata.tables.items():
                sql_str = f"show table status from {config.DB_SCHEMA_MD} where name=:table_name"
                row_data = session.execute(sql_str, params={'table_name': table_name}).first()
                if row_data is None:
                    continue
                if row_data[1].lower() == 'myisam':
                    continue

                logger.info('修改 %s 表引擎为 MyISAM', table_name)
                sql_str = "ALTER TABLE %s ENGINE = MyISAM" % table_name
                session.execute(sql_str)

    logger.info("所有表结构建立完成")


def dynamic_load_table_model():
    """
    动态加载数据库中所有表model，保存到 table_model_dic
    :return:
    """
    logger.info('动态加载数据库 %s 的表', config.DB_SCHEMA_MD)
    metadata = MetaData(engine_md)
    sql_str = "select TABLE_NAME from information_schema.TABLES where table_schema=:table_schema"
    with with_db_session(engine_md) as session:
        table_name_list = [row[0] for row in session.execute(
            sql_str, params={'table_schema': config.DB_SCHEMA_MD}).fetchall()]
    table_name_list_len = len(table_name_list)
    for num, table_name in enumerate(table_name_list, start=1):
        model = Table(table_name, metadata, autoload=True)
        table_model_dic[table_name] = model
        logger.debug('%d/%d) load table %s', num, table_name_list_len, table_name)


if __name__ == "__main__":
    init(True)
    dynamic_load_table_model()



#! /usr/bin/env python
# -*- coding:utf-8 -*-
"""
@author  : MG
@Time    : 2018/10/17 16:19
@File    : handler.py
@contact : mmmaaaggg@163.com
@desc    : 
"""
import logging
from ibats_common.common import PeriodType
from prodconpattern import ProducerConsumer
from ibats_common.utils.mess import datetime_2_str, STR_FORMAT_DATETIME2
from datetime import datetime
from sqlalchemy import Table, MetaData
from sqlalchemy.orm import sessionmaker
from ibats_common.utils.redis import get_redis, get_channel
import json
from ibats_bitmex_feeder.config import config
from ibats_bitmex_feeder.backend import engine_md
logger = logging.getLogger()


class BaseHandler:

    def __init__(self, name):
        self.name = name
        self.logger = logging.getLogger(__name__)


class SimpleHandler(BaseHandler):

    def handle(self, msg: dict):
        logger.debug(msg)


class DBHandler(BaseHandler):

    def __init__(self, period, db_model=None, table_name=None):
        """
        接收数据，并插入到对应数据库
        :param period: 周期
        :param db_model: 模型
        :param table_name: 表名
        """
        self.period = period
        if db_model is not None:
            self.table_name = db_model.__tablename__
            self.md_orm_table = db_model.__table__
        elif table_name is not None:
            self.table_name = table_name
            self.md_orm_table = Table(table_name, MetaData(engine_md), autoload=True)
        else:
            raise ValueError('db_model, table_name 需要至少设置一个参数')
        BaseHandler.__init__(self, 'DB[%s]' % self.table_name)
        self.session_maker = sessionmaker(bind=engine_md)
        self.session = None
        self.logger = logging.getLogger(f'{__name__}->{self.table_name}')
        self.md_orm_table_insert = self.md_orm_table.insert(on_duplicate_key_update=True)

    def handle(self, msg: dict):
        self.save_md(msg)
        self.logger.debug('invoke save_md %s', msg)

    @ProducerConsumer(threshold=2000, interval=60, pass_arg_list=True, is_class_method=True)
    def save_md(self, data_dic_list):
        """
        保存md数据到数据库及文件
        :param data_dic_list:
        :param session:
        :return:
        """
        # 仅调试使用
        # if self.table_name == 'md_min60_bc':
        # self.logger.info('%d data will be save to %s', len(data_dic_list), self.table_name)

        if data_dic_list is None or len(data_dic_list) == 0:
            self.logger.warning("data_dic_list 为空")
            return

        md_count = len(data_dic_list)
        # 保存到数据库
        if self.session is None:
            self.session = self.session_maker()
        try:
            self.session.execute(self.md_orm_table_insert, data_dic_list)
            self.session.commit()
            self.logger.info('%d 条实时行情 -> %s 完成', md_count, self.table_name)
        except:
            self.logger.exception('%d 条实时行情 -> %s 失败', md_count, self.table_name)


class PublishHandler(BaseHandler):

    def __init__(self, period):
        BaseHandler.__init__(self, name=self.__class__.__name__)
        self.period = period
        self.r = get_redis(config.REDIS_INFO_DIC['REDIS_HOST'], config.REDIS_INFO_DIC['REDIS_PORT'])

    def handle(self, msg: dict):
        """
        收到数据后，tick数据直接发送，
        channel：md.market.tick.pair
        channel：md.market.min1.pair 每个分钟时点切换时，发送一次分钟线数据
        例如：
        md.huobi.tick.ethusdt
        md.huobi.1min.ethusdt
        通过 redis-cli 可以 PUBSUB CHANNELS 查阅活跃的频道
        PSUBSCRIBE pattern [pattern ...]  查看频道内容
        SUBSCRIBE channel [channel ...]  查看频道内容
        :param msg:
        :return:
        """
        # TODO: 设定一个定期检查机制，只发送订阅的品种，降低网络负载
        md_str = json.dumps(msg)
        symbol = msg['symbol']
        channel = get_channel(config.MARKET_NAME, self.period, symbol)
        self.r.publish(channel, md_str)



#! /usr/bin/env python
# -*- coding:utf-8 -*-
"""
@author  : MG
@Time    : 2018/9/27 10:40
@File    : feeder.py
@contact : mmmaaaggg@163.com
@desc    : 
"""
from threading import Thread
from ibats_bitmex_feeder.backend.bitmex_ws import BitMexWS, TableNoAuth
import bitmex
from pandas._libs.tslibs.timestamps import Timestamp
from sqlalchemy import func
from ibats_bitmex_feeder.backend.check import check_redis
from ibats_bitmex_feeder.config import config
from ibats_bitmex_feeder.utils.mess import load_df_against_pagination, load_list_against_pagination
from sqlalchemy.types import String, Date, DateTime, Time, Integer, Boolean
from sqlalchemy.dialects.mysql import DOUBLE, TIMESTAMP
from ibats_common.utils.db import bunch_insert_on_duplicate_update, with_db_session
from ibats_bitmex_feeder.backend import engine_md
import time
from ibats_common.utils.mess import try_n_times, date_2_str, datetime_2_str
import logging
from datetime import datetime, timedelta
from ibats_bitmex_feeder.backend.orm import MDMin1, MDMin1Temp, MDDaily, MDDailyTemp, MDHour1, MDHour1Temp, \
    MDMin5, MDMin5Temp, BaseModel, INSTRUMENT_INFO_TABLE_NAME
from ibats_bitmex_feeder.backend.handler import DBHandler, PublishHandler, SimpleHandler
from ibats_common.common import PeriodType

logger = logging.getLogger()
DTYPE_INSTRUMENT = {
    'symbol': String(20),
    'rootSymbol': String(10),
    'state': String(20),
    'typ': String(10),
    'listing': DateTime,
    'front': DateTime,
    'expiry': DateTime,
    'settle': DateTime,
    'relistInterval': DateTime,
    'inverseLeg': String(20),
    'sellLeg': String(20),
    'buyLeg': String(20),
    'optionStrikePcnt': DOUBLE,
    'optionStrikeRound': DOUBLE,
    'optionStrikePrice': DOUBLE,
    'optionMultiplier': DOUBLE,
    'positionCurrency': String(20),
    'underlying': String(20),
    'quoteCurrency': String(20),
    'underlyingSymbol': String(20),
    'reference': String(20),
    'referenceSymbol': String(20),
    'calcInterval': DateTime,
    'publishInterval': DateTime,
    'publishTime': DateTime,
    'maxOrderQty': DOUBLE,
    'maxPrice': DOUBLE,
    'lotSize': DOUBLE,
    'tickSize': DOUBLE,
    'multiplier': DOUBLE,
    'settlCurrency': String(20),
    'underlyingToPositionMultiplier': DOUBLE,
    'underlyingToSettleMultiplier': DOUBLE,
    'quoteToSettleMultiplier': DOUBLE,
    'isQuanto': Boolean,
    'isInverse': Boolean,
    'initMargin': DOUBLE,
    'maintMargin': DOUBLE,
    'riskLimit': DOUBLE,
    'riskStep': DOUBLE,
    'limit': DOUBLE,
    'capped': Boolean,
    'taxed': Boolean,
    'deleverage': Boolean,
    'makerFee': DOUBLE,
    'takerFee': DOUBLE,
    'settlementFee': DOUBLE,
    'insuranceFee': DOUBLE,
    'fundingBaseSymbol': String(20),
    'fundingQuoteSymbol': String(20),
    'fundingPremiumSymbol': String(20),
    'fundingTimestamp': DateTime,
    'fundingInterval': DateTime,
    'fundingRate': DOUBLE,
    'indicativeFundingRate': DOUBLE,
    'rebalanceTimestamp': DateTime,
    'rebalanceInterval': DateTime,
    'openingTimestamp': DateTime,
    'closingTimestamp': DateTime,
    'sessionInterval': DateTime,
    'prevClosePrice': DOUBLE,
    'limitDownPrice': DOUBLE,
    'limitUpPrice': DOUBLE,
    'bankruptLimitDownPrice': DOUBLE,
    'bankruptLimitUpPrice': DOUBLE,
    'prevTotalVolume': DOUBLE,
    'totalVolume': DOUBLE,
    'volume': DOUBLE,
    'volume24h': DOUBLE,
    'prevTotalTurnover': DOUBLE,
    'totalTurnover': DOUBLE,
    'turnover': DOUBLE,
    'turnover24h': DOUBLE,
    'homeNotional24h': DOUBLE,
    'foreignNotional24h': DOUBLE,
    'prevPrice24h': DOUBLE,
    'vwap': DOUBLE,
    'highPrice': DOUBLE,
    'lowPrice': DOUBLE,
    'lastPrice': DOUBLE,
    'lastPriceProtected': DOUBLE,
    'lastTickDirection': String(20),
    'lastChangePcnt': DOUBLE,
    'bidPrice': DOUBLE,
    'midPrice': DOUBLE,
    'askPrice': DOUBLE,
    'impactBidPrice': DOUBLE,
    'impactMidPrice': DOUBLE,
    'impactAskPrice': DOUBLE,
    'hasLiquidity': Boolean,
    'openInterest': DOUBLE,
    'openValue': DOUBLE,
    'fairMethod': String(20),
    'fairBasisRate': DOUBLE,
    'fairBasis': DOUBLE,
    'fairPrice': DOUBLE,
    'markMethod': String(30),
    'markPrice': DOUBLE,
    'indicativeTaxRate': DOUBLE,
    'indicativeSettlePrice': DOUBLE,
    'optionUnderlyingPrice': DOUBLE,
    'settledPrice': DOUBLE,
    'timestamp': DateTime
}


class MDFeeder(Thread):
    """接受订阅数据想redis发送数据"""

    def __init__(self, do_init_symbols=False, do_fill_history=False):
        super().__init__(name='Feeder', daemon=True)
        self.endpoint = "https://testnet.bitmex.com/api/v1" if config.TEST_NET else "https://www.bitmex.com/api/v1"
        self.ws = BitMexWS(endpoint=self.endpoint, api_key=None, api_secret=None)
        self.api = bitmex.bitmex(test=config.TEST_NET)
        self.api_auth = bitmex.bitmex(test=config.TEST_NET,
                                      api_key=config.EXCHANGE_PUBLIC_KEY, api_secret=config.EXCHANGE_SECRET_KEY)
        self.do_init_symbols = do_init_symbols
        self.do_fill_history = do_fill_history
        self.logger = logging.getLogger(self.__class__.__name__)
        self.is_working = True
        # self.heart_beat = HeartBeatHandler()

    def init(self):
        """
        初始化，订阅行情
        默认1分钟、1小时、1日
        :return:
        """

        table_name = INSTRUMENT_INFO_TABLE_NAME
        if self.do_init_symbols or not engine_md.has_table(table_name):
            # 获取有效的交易对信息保存（更新）数据库
            ret_df = load_df_against_pagination(self.api.Instrument.Instrument_get)

            for key, val in DTYPE_INSTRUMENT.items():
                if val in (DateTime, TIMESTAMP):
                    ret_df[key] = ret_df[key].apply(
                        lambda x: x.strftime('%Y-%m-%d %H-%M-%S') if isinstance(x, Timestamp) else None)
            data_count = bunch_insert_on_duplicate_update(
                ret_df, table_name, engine_md, dtype=DTYPE_INSTRUMENT,
                myisam_if_create_table=True, primary_keys=['symbol'])
            self.logger.info('更新 instrument 信息 %d 条', data_count)
            symbol_set = set(list(ret_df[ret_df['state'] == 'Open']['symbol']))
        else:
            sql_str = f"select symbol from {table_name} where state='Open'"
            with with_db_session(engine_md) as session:
                symbol_set = set(list(session.execute(sql_str).fetchall()))

        # symbol_list_len = len(symbol_set)
        # self.logger.info('订阅实时行情 %d 项：%s', symbol_list_len, symbol_set)

        # 订阅 instrument
        self.ws.register_handler(
            'db', handler=DBHandler(period=PeriodType.Min1, db_model=MDMin1), table=TableNoAuth.tradeBin1m)
        self.ws.register_handler(
            'db', handler=DBHandler(period=PeriodType.Min5, db_model=MDMin5), table=TableNoAuth.tradeBin5m)
        self.ws.register_handler(
            'db', handler=DBHandler(period=PeriodType.Hour1, db_model=MDHour1), table=TableNoAuth.tradeBin1h)
        self.ws.register_handler(
            'db', handler=DBHandler(period=PeriodType.Day1, db_model=MDDaily), table=TableNoAuth.tradeBin1d)

        # 数据redis广播
        if config.REDIS_PUBLISHER_HANDLER_ENABLE and check_redis():
            self.ws.register_handler('publisher', PublishHandler(period=PeriodType.Min1), table=TableNoAuth.tradeBin1m)
            self.ws.register_handler('publisher', PublishHandler(period=PeriodType.MDMin5),
                                     table=TableNoAuth.tradeBin5m)
            self.ws.register_handler('publisher', PublishHandler(period=PeriodType.MDHour1),
                                     table=TableNoAuth.tradeBin1h)
            self.ws.register_handler('publisher', PublishHandler(period=PeriodType.MDDaily),
                                     table=TableNoAuth.tradeBin1d)

        # Heart Beat
        # self.hb.register_handler(self.heart_beat)
        # logger.info('注册 %s 处理句柄', self.heart_beat.name)
        #
        # server_datetime = self.get_server_datetime()
        # logger.info("api.服务期时间 %s 与本地时间差： %f 秒",
        #             server_datetime, (datetime.now() - server_datetime).total_seconds())
        # self.check_state()

    def stop(self):
        # 关闭所有 ws
        self.ws.exit()
        self.logger.info('结束订阅')

    def run(self):
        self.logger.info('启动')
        # 补充历史数据
        if self.do_fill_history:
            self.logger.info('开始补充历史数据')
            self.fill_history()
        while self.is_working:
            time.sleep(5)

    def fill_history(self, periods=['1m', '5m', '1h', '1d']):
        for period in periods:
            if period == '1m':
                model_tot, model_tmp = MDMin1, MDMin1Temp
            elif period == '5m':
                model_tot, model_tmp = MDMin5, MDMin5Temp
            elif period == '1h':
                model_tot, model_tmp = MDHour1, MDHour1Temp
            elif period == '1d':
                model_tot, model_tmp = MDDaily, MDDailyTemp
            else:
                self.logger.warning(f'{period} 不是有效的周期')
                raise ValueError(f'{period} 不是有效的周期')

            self.fill_history_period(period, model_tot, model_tmp)

    def fill_history_period(self, period, model_tot: BaseModel, model_tmp: BaseModel):
        """
        根据数据库中的支持 symbol 补充历史数据
        api.Trade.Trade_get(symbol='XBTUSD').result()
        :param period:
        :param model_tot:
        :param model_tmp:
        :return:
        """
        if period not in {'1m', '5m', '1h', '1d'}:
            logger.error(f'{period} 不是有效的周期')
            raise ValueError(f'{period} 不是有效的周期')

        # 查找有效的 symbol
        table_name = 'bitmex_instrument'
        sql_str = f"select symbol from {table_name} where state='Open'"
        with with_db_session(engine_md) as session:
            symbol_set = set([row[0] for row in session.execute(sql_str).fetchall()])
            pair_datetime_latest_dic = dict(
                session.query(
                    model_tmp.symbol, func.max(model_tmp.timestamp)
                ).group_by(model_tmp.symbol).all()
            )

        # 循环获取每一个交易对的历史数据
        symbol_set_len = len(symbol_set)
        for num, symbol in enumerate(symbol_set):
            if symbol in pair_datetime_latest_dic:
                datetime_latest = pair_datetime_latest_dic[symbol]
                datetime_start = datetime_latest + timedelta(minutes=1)

                # 设置 startTime
                if period == '1m':
                    size = int((datetime.now() - datetime_start).total_seconds() / 60)
                elif period == '5m':
                    size = int((datetime.now() - datetime_start).total_seconds() / 300)
                elif period == '1h':
                    size = int((datetime.now() - datetime_start).total_seconds() / 3600)
                else:  # if period == '1d':
                    size = (datetime.now() - datetime_start).days

                start_time = datetime_2_str(datetime_start, '%Y-%m-%d %H:%M')
            else:
                size = 500
                start_time = None

            if size <= 0:
                continue

            ret_list = self.get_kline(symbol, period, start_time=start_time)
            if ret_list is None or len(ret_list) == 0:
                continue
            ret_list_len = len(ret_list)
            for data_dic in ret_list:
                data_dic['timestamp'] = datetime_2_str(data_dic['timestamp'])
            # data_count = bunch_insert_on_duplicate_update(ret_df, model_tmp.__tablename__, engine_md)
            self._save_md(ret_list, symbol, model_tot, model_tmp)
            logger.info('%d/%d) %s %s 更新 %d 数据成功', num, symbol_set_len, symbol, period, ret_list_len)

    @try_n_times(5, sleep_time=5, logger=logger)
    def get_kline(self, symbol, period, start_time=None) -> list:
        # ret_df = self.api.Trade.Trade_getBucketed(symbol=symbol, binSize=period, startTime=start_time).result()[0]
        ret_list = load_list_against_pagination(self.api.Trade.Trade_getBucketed,
                                                symbol=symbol, binSize=period, startTime=start_time)
        return ret_list

    def _save_md(self, data_dic_list, symbol, model_tot: MDMin1, model_tmp: MDMin1Temp):
        """
        保存md数据到数据库及文件
        :param data_dic_list:
        :param symbol:
        :param model_tot:
        :param model_tmp:
        :return:
        """

        if data_dic_list is None:
            self.logger.warning("data_dic_list 为空")
            return

        md_count = len(data_dic_list)
        if md_count == 0:
            self.logger.warning("data_dic_list len is 0")
            return
        table_name = model_tot.__tablename__
        # 保存到数据库
        with with_db_session(engine_md) as session:
            try:
                # 插入数据到临时表
                session.execute(model_tmp.__table__.insert(on_duplicate_key_update=True), data_dic_list)
                self.logger.info('%d 条 %s 历史数据 -> %s 完成', md_count, symbol, table_name)
                # 将临时表数据导入总表
                sql_str = f"""insert into `{table_name}` 
                    select * from `{model_tmp.__tablename__}` where symbol=:symbol ON DUPLICATE KEY UPDATE 
                    open=VALUES(open), high=VALUES(high), low=VALUES(low), close=VALUES(close),
                    trades=VALUES(trades), volume=VALUES(volume), vwap=VALUES(vwap), lastSize=VALUES(lastSize),
                    turnover=VALUES(turnover), homeNotional=VALUES(homeNotional), 
                    foreignNotional=VALUES(foreignNotional)"""
                session.execute(sql_str, params={"symbol": symbol})
                # 删除临时表数据，仅保留最新的一条记录
                datetime_latest = session.query(
                    func.max(model_tmp.timestamp).label('ts_latest')
                ).filter(
                    model_tmp.symbol == symbol
                ).scalar()
                # issue:
                # https://stackoverflow.com/questions/9882358/how-to-delete-rows-from-a-table-using-an-sqlalchemy-query-without-orm
                delete_count = session.query(model_tmp).filter(
                    model_tmp.symbol == symbol,
                    model_tmp.timestamp < datetime_latest
                ).delete()
                self.logger.debug('%d 条 %s 历史数据被清理，最新数据日期 %s', delete_count, symbol, datetime_latest)
                session.commit()
            except:
                self.logger.exception('%d 条 %s 数据-> %s 失败', md_count, symbol, table_name)


def start_feeder(init_symbols=False, do_fill_history=False) -> MDFeeder:
    feeder = MDFeeder(do_init_symbols=init_symbols, do_fill_history=do_fill_history)
    feeder.init()
    feeder.start()
    return feeder



#! /usr/bin/env python
# -*- coding:utf-8 -*-
"""
@author  : MG
@Time    : 2018/10/17 15:54
@File    : bitmex_ws.py
@contact : mmmaaaggg@163.com
@desc    : 根据官网资料 https://github.com/BitMEX/api-connectors/tree/master/official-ws/python 进一步修改
支持订阅全部合约

# You can sub to orderBookL2 for all levels, or orderBook10 for top 10 levels & save bandwidth
# 下面的订阅主题是无需身份验证
# "announcement",// 网站公告
# "chat",        // Trollbox 聊天室
# "connected",   // 已连接用户/机器人的统计数据
# "funding",     // 掉期产品的资金费率更新 每个资金时段发送（通常是8小时）
# "instrument",  // 产品更新，包括交易量以及报价
# "insurance",   // 每日保险基金的更新
# "liquidation", // 进入委托列表的强平委托
# "orderBookL2", // 完整的 level 2 委托列表
# "orderBook10", //  前10层的委托列表，用传统的完整委托列表推送
# "publicNotifications", // 全系统的告示（用于段时间的消息）
# "quote",       // 最高层的委托列表
# "quoteBin1m",  // 每分钟报价数据
# "quoteBin5m",  // 每5分钟报价数据
# "quoteBin1h",  // 每个小时报价数据
# "quoteBin1d",  // 每天报价数据
# "settlement",  // 结算信息
# "trade",       // 实时交易
# "tradeBin1m",  // 每分钟交易数据
# "tradeBin5m",  // 每5分钟交易数据
# "tradeBin1h",  // 每小时交易数据
# "tradeBin1d",  // 每天交易数据

# 下列主题要求进行身份验证︰
# "affiliate",   // 邀请人状态，已邀请用户及分红比率
# "execution",   // 个别成交，可能是多个成交
# "order",       // 你委托的更新
# "margin",      // 你账户的余额和保证金要求的更新
# "position",    // 你仓位的更新
# "privateNotifications", // 个人的通知，现时并未使用
# "transact"     // 资金提存更新
# "wallet"       // 比特币余额更新及总提款存款

"""
import websocket
import threading
import traceback
from time import sleep
import json
import logging
import urllib
from util.api_key import generate_nonce, generate_signature
from collections import defaultdict


class TableNoAuth:
    announcement = "announcement"  # 网站公告
    chat = "chat"  # Trollbox 聊天室
    connected = "connected"  # 已连接用户/机器人的统计数据
    funding = "funding"  # 掉期产品的资金费率更新 每个资金时段发送（通常是8小时）
    instrument = "instrument"  # 产品更新，包括交易量以及报价
    insurance = "insurance"  # 每日保险基金的更新
    liquidation = "liquidation"  # 进入委托列表的强平委托
    orderBookL2 = "orderBookL2"  # 完整的 level 2 委托列表
    orderBook10 = "orderBook10"  # 前10层的委托列表，用传统的完整委托列表推送
    publicNotifications = "publicNotifications"  # 全系统的告示（用于段时间的消息）
    quote = "quote"  # 最高层的委托列表
    quoteBin1m = "quoteBin1m"  # 每分钟报价数据
    quoteBin5m = "quoteBin5m"  # 每5分钟报价数据
    quoteBin1h = "quoteBin1h"  # 每个小时报价数据
    quoteBin1d = "quoteBin1d"  # 每天报价数据
    settlement = "settlement"  # 结算信息
    trade = "trade"  # 实时交易
    tradeBin1m = "tradeBin1m"  # 每分钟交易数据
    tradeBin5m = "tradeBin5m"  # 每5分钟交易数据
    tradeBin1h = "tradeBin1h"  # 每小时交易数据
    tradeBin1d = "tradeBin1d"  # 每天交易数据


class TableAuth:
    affiliate = "affiliate"  # 邀请人状态，已邀请用户及分红比率
    execution = "execution"  # 个别成交，可能是多个成交
    order = "order"  # 你委托的更新
    margin = "margin"  # 你账户的余额和保证金要求的更新
    position = "position"  # 你仓位的更新
    privateNotifications = "privateNotifications"  # 个人的通知，现时并未使用
    transact = "transact"  # 资金提存更新
    wallet = "wallet"  # 比特币余额更新及总提款存款


DEFAULT_SUBSCRIPTIONS = [
    TableNoAuth.quote, TableNoAuth.tradeBin1d, TableNoAuth.tradeBin5m, TableNoAuth.tradeBin1h, TableNoAuth.tradeBin1d]


# Naive implementation of connecting to BitMEX websocket for streaming realtime data.
# The Marketmaker still interacts with this as if it were a REST Endpoint, but now it can get
# much more realtime data without polling the hell out of the API.
#
# The Websocket offers a bunch of data as raw properties right on the object.
# On connect, it synchronously asks for a push of all this data then returns.
# Right after, the MM can start using its data. It will be updated in realtime, so the MM can
# poll really often if it wants.
class BitMexWS:
    # Don't grow a table larger than this amount. Helps cap memory usage.
    MAX_TABLE_LEN = 200

    def __init__(self, endpoint, api_key=None, api_secret=None, subscriptions=DEFAULT_SUBSCRIPTIONS):
        """Connect to the websocket and initialize data stores."""
        self.logger = logging.getLogger(__name__)
        self.logger.debug("Initializing WebSocket.")
        self.endpoint = endpoint
        self.table_handler = defaultdict(lambda: defaultdict(dict))

        if api_key is not None and api_secret is None:
            raise ValueError('api_secret is required if api_key is provided')
        if api_key is None and api_secret is not None:
            raise ValueError('api_key is required if api_secret is provided')

        self.api_key = api_key
        self.api_secret = api_secret

        self.data = {}
        self.keys = {}
        self.exited = False
        self.subscriptions = subscriptions

    def connect(self):
        # We can subscribe right in the connection querystring, so let's build that.
        # Subscribe to all pertinent endpoints
        ws_url = self.__get_url()
        self.logger.info("Connecting to %s" % ws_url)
        self.__connect(ws_url)
        self.logger.info('Connected to WS.')

        # Connected. Wait for partials
        # self.__wait_for_symbol(symbol)
        if self.api_key:
            self.__wait_for_account()
        self.logger.info('Got all market data. Starting.')

    def exit(self):
        """Call this to exit - will close websocket."""
        self.exited = True
        self.ws.close()

    def register_handler(self, name, handler, table='quoteBin1m', actions={'partial', 'insert'}):
        """向ws messsage 事件中注册事件处理具备"""
        for action in actions:
            self.table_handler[table][action][name] = handler
            logging.info('注册事件句柄 %s -> %s -> %s', table, action, name)

    #
    # End Public Methods
    #

    def __connect(self, ws_url):
        """Connect to the websocket in a thread."""
        self.logger.debug("Starting thread")

        self.ws = websocket.WebSocketApp(ws_url,
                                         on_message=self.__on_message,
                                         on_close=self.__on_close,
                                         on_open=self.__on_open,
                                         on_error=self.__on_error,
                                         header=self.__get_auth())

        self.wst = threading.Thread(target=lambda: self.ws.run_forever())
        self.wst.daemon = True
        self.wst.start()
        self.logger.debug("Started thread")

        # Wait for connect before continuing
        conn_timeout = 5
        while not self.ws.sock or not self.ws.sock.connected and conn_timeout:
            sleep(1)
            conn_timeout -= 1
        if not conn_timeout:
            self.logger.error("Couldn't connect to WS! Exiting.")
            self.exit()
            raise websocket.WebSocketTimeoutException("Couldn't connect to WS! Exiting.")

    def __get_auth(self):
        """Return auth headers. Will use API Keys if present in settings."""
        if self.api_key:
            self.logger.info("Authenticating with API Key.")
            # To auth to the WS using an API key, we generate a signature of a nonce and
            # the WS API endpoint.
            nonce = generate_nonce()
            return [
                "api-nonce: " + str(nonce),
                "api-signature: " + generate_signature(self.api_secret, 'GET', '/realtime', nonce, ''),
                "api-key:" + self.api_key
            ]
        else:
            self.logger.info("Not authenticating.")
            return []

    def __get_url(self):
        """
        Generate a connection URL. We can define subscriptions right in the querystring.
        Most subscription topics are scoped by the symbol we're listening to.
        """
        url_parts = list(urllib.parse.urlparse(self.endpoint))
        url_parts[0] = url_parts[0].replace('http', 'ws')
        url_parts[2] = "/realtime?subscribe={}".format(','.join(self.subscriptions))
        return urllib.parse.urlunparse(url_parts)

    def __wait_for_account(self):
        """On subscribe, this data will come down. Wait for it."""
        # Wait for the keys to show up from the ws
        # while not {'margin', 'position', 'order', 'orderBookL2'} <= set(self.data):
        while not {'orderBookL2'} <= set(self.data):
            sleep(0.1)

    def __wait_for_symbol(self):
        """On subscribe, this data will come down. Wait for it."""
        while not {'instrument', 'trade', 'quote'} <= set(self.data):
            sleep(0.1)

    def __send_command(self, command, args=None):
        """Send a raw command."""
        if args is None:
            args = []
        self.ws.send(json.dumps({"op": command, "args": args}))

    def __on_message(self, ws, message):
        """Handler for parsing WS messages."""
        message = json.loads(message)
        # self.logger.debug(json.dumps(message))

        table = message['table'] if 'table' in message else None
        action = message['action'] if 'action' in message else None
        try:
            if 'subscribe' in message:
                self.logger.debug("Subscribed to %s.", message['subscribe'])
            elif action:

                if table not in self.data:
                    self.data[table] = []

                data = message['data']
                # There are four possible actions from the WS:
                # 'partial' - full table image
                # 'insert'  - new row
                # 'update'  - update row
                # 'delete'  - delete row
                if action == 'partial':
                    self.logger.debug("%s: partial", table)
                    self.data[table] += data
                    # Keys are communicated on partials to let you know how to uniquely identify
                    # an item. We use it for updates.
                    self.keys[table] = message['keys']
                elif action == 'insert':
                    self.logger.debug('%s: inserting %s', table, data)
                    self.data[table] += data

                    # Limit the max length of the table to avoid excessive memory usage.
                    # Don't trim orders because we'll lose valuable state if we do.
                    if table not in ['order', 'orderBookL2'] and len(self.data[table]) > BitMexWS.MAX_TABLE_LEN:
                        self.data[table] = self.data[table][int(BitMexWS.MAX_TABLE_LEN / 2):]

                elif action == 'update':
                    self.logger.debug('%s: updating %s', table, data)
                    # Locate the item in the collection and update it.
                    for updateData in data:
                        item = findItemByKeys(self.keys[table], self.data[table], updateData)
                        if not item:
                            return  # No item found to update. Could happen before push
                        item.update(updateData)
                        # Remove cancelled / filled orders
                        if table == 'order' and item['leavesQty'] <= 0:
                            self.data[table].remove(item)
                elif action == 'delete':
                    self.logger.debug('%s: deleting %s' % (table, data))
                    # Locate the item in the collection and remove it.
                    for deleteData in data:
                        item = findItemByKeys(self.keys[table], self.data[table], deleteData)
                        self.data[table].remove(item)
                else:
                    raise Exception("Unknown action: %s" % action)

                # 处理事件句柄
                for name, handler in self.table_handler[table][action].items():
                    self.logger.debug('%s->%s->%s', table, action, name)
                    for data_dic in data:
                        handler(data_dic)
        except:
            self.logger.error(traceback.format_exc())

    def __on_error(self, ws, error):
        """Called on fatal websocket errors. We exit on these."""
        if not self.exited:
            self.logger.error("Error : %s" % error)
            raise websocket.WebSocketException(error)

    def __on_open(self, ws):
        """Called when the WS opens."""
        self.logger.debug("Websocket Opened.")

    def __on_close(self, ws):
        """Called on websocket close."""
        self.logger.info('Websocket Closed')


# Utility method for finding an item in the store.
# When an update comes through on the websocket, we need to figure out which item in the array it is
# in order to match that item.
#
# Helpfully, on a data push (or on an HTTP hit to /api/v1/schema), we have a "keys" array. These are the
# fields we can use to uniquely identify an item. Sometimes there is more than one, so we iterate through all
# provided keys.
def findItemByKeys(keys, table, matchData):
    for item in table:
        matched = True
        for key in keys:
            if item[key] != matchData[key]:
                matched = False
                break
        if matched:
            return item



#! /usr/bin/env python
# -*- coding:utf-8 -*-
"""
@author  : MG
@Time    : 2018/6/6 9:57
@File    : check.py
@contact : mmmaaaggg@163.com
@desc    : 用于对系统配置的环境进行检测，检查是否环境可用，包括mysql、redis等
"""
import threading
import json
import time
import logging
from ibats_common.common import PeriodType
from ibats_common.utils.mess import bytes_2_str
from ibats_common.utils.redis import get_channel, get_redis
from ibats_bitmex_feeder.config import config
logger = logging.getLogger()
_signal = {}


def _timer(channel):
    global _signal
    count = 0
    r = get_redis(config.REDIS_INFO_DIC['REDIS_HOST'], config.REDIS_INFO_DIC['REDIS_PORT'])
    while not _signal['redis']:
        md_str = json.dumps({"message": "Hello World!!", "count": count})
        r.publish(channel, md_str)
        logger.debug("发布成功 %s", md_str)
        count += 1
        if count >= 15:
            break
        time.sleep(1)


def check_redis():
    """
    检测redis是否可以正常工作
    """
    global _signal
    instrument_id = 'rb1805'
    channel = get_channel(market=config.MARKET_NAME, period=PeriodType.Year1, symbol=instrument_id)
    logger.info('测试 Channel：%s', channel)
    _signal['redis'] = False

    timer_t = threading.Thread(target=_timer, args=(channel,))
    timer_t.start()

    def _receiver(_channel):
        # 接收订阅的行情，成功接收后退出
        global _signal
        redis_client = get_redis(config.REDIS_INFO_DIC['REDIS_HOST'], config.REDIS_INFO_DIC['REDIS_PORT'])
        try:
            pub_sub = redis_client.pubsub()
            pub_sub.psubscribe(_channel)
            for item in pub_sub.listen():
                logger.debug("接收成功 %s", item)
                if item['type'] == 'pmessage':
                    md_dic_str = bytes_2_str(item['data'])
                    md_dic = json.loads(md_dic_str)
                    if "message" in md_dic and "count" in md_dic:
                        _signal['redis'] = True
                        logger.debug("接收到消息")
                        break
        except:
            logger.exception('Redis 检测时发现异常，可能是由于redis没有启动')
            pass

    receiver_t = threading.Thread(target=_receiver, args=(channel,))
    receiver_t.start()

    for n in range(20):
        if _signal['redis']:
            logging.debug("检测redis %d %s", n, _signal['redis'])
            timer_t.join(1)
            break
        time.sleep(1)
    else:
        logger.error("redis 检测未通过")

    return _signal['redis']


def check():
    ok_list = []
    is_ok = check_redis()
    ok_list.append(is_ok)
    if is_ok:
        logger.info("redis 检测成功")

    return all(ok_list)


if __name__ == "__main__":
    is_all_ok = check()
    logger.info("全部检测完成，%s", is_all_ok)



#! /usr/bin/env python
# -*- coding:utf-8 -*-
"""
@author  : MG
@Time    : 2018/6/12 13:47
@File    : __init__.py.py
@contact : mmmaaaggg@163.com
@desc    : 
"""
from sqlalchemy import create_engine
from ibats_bitmex_feeder.config import config

engines = {key: create_engine(url) for key, url in config.DB_URL_DIC.items()}

engine_md = engines[config.DB_SCHEMA_MD]



#! /usr/bin/env python
# -*- coding:utf-8 -*-
"""
@author  : MG
@Time    : 2018/9/27 10:49
@File    : config.py
@contact : mmmaaaggg@163.com
@desc    : 
"""
import logging

logger = logging.getLogger()


class ConfigBase:

    # 交易所名称
    MARKET_NAME = 'bitmex'

    # api configuration
    # https://testnet.bitmex.com/app/apiKeys
    TEST_NET = True
    EXCHANGE_PUBLIC_KEY = "K5DaKlClbXg_TQn5lEGOswd8"
    EXCHANGE_SECRET_KEY = "QQwPpUpCUcJwtqFIsDXevMqhEPUM3eanZUnzlSpYGqaLIbph"

    # mysql db info
    DB_HANDLER_ENABLE = True
    DB_SCHEMA_MD = 'md_bitmex'
    DB_URL_DIC = {
        DB_SCHEMA_MD: 'mysql://mg:Dcba1234@localhost/' + DB_SCHEMA_MD
    }

    # redis info
    REDIS_PUBLISHER_HANDLER_ENABLE = False
    REDIS_INFO_DIC = {'REDIS_HOST': 'localhost',  # '192.168.239.131'
                      'REDIS_PORT': '6379',
                      }

    # evn configuration
    LOG_FORMAT = '%(asctime)s %(levelname)s %(name)s %(filename)s.%(funcName)s:%(lineno)d|%(message)s'

    def __init__(self):
        """
        初始化一些基本配置信息
        """

        # log settings
        logging_config = dict(
            version=1,
            formatters={
                'simple': {
                    'format': ConfigBase.LOG_FORMAT}
            },
            handlers={
                'file_handler':
                    {
                        'class': 'logging.handlers.RotatingFileHandler',
                        'filename': 'logger.log',
                        'maxBytes': 1024 * 1024 * 10,
                        'backupCount': 5,
                        'level': 'DEBUG',
                        'formatter': 'simple',
                        'encoding': 'utf8'
                    },
                'console_handler':
                    {
                        'class': 'logging.StreamHandler',
                        'level': 'DEBUG',
                        'formatter': 'simple'
                    }
            },

            root={
                'handlers': ['console_handler', 'file_handler'],  #
                'level': logging.DEBUG,
            }
        )
        # 设置日志输出级别
        # logging.getLogger('sqlalchemy.engine').setLevel(logging.WARN)
        # logging.getLogger('requests.packages.urllib3.connectionpool').setLevel(logging.WARNING)
        # logging.getLogger('DBHandler->md_min1_tick_bc').setLevel(logging.INFO)
        # logging.getLogger('DBHandler->md_min1_bc').setLevel(logging.INFO)
        # logging.getLogger('DBHandler->md_min60_bc').setLevel(logging.INFO)
        # logging.getLogger('DBHandler->md_daily_bc').setLevel(logging.INFO)
        # logging.getLogger('MDFeeder').setLevel(logging.INFO)
        # logging.getLogger('md_min1_bc').setLevel(logging.INFO)
        logging.getLogger('urllib3.connectionpool').setLevel(logging.INFO)
        logging.getLogger('bravado_core.model').setLevel(logging.INFO)
        logging.getLogger('bravado.client').setLevel(logging.INFO)
        logging.getLogger('bravado_core.resource').setLevel(logging.INFO)
        logging.getLogger('swagger_spec_validator.ref_validators').setLevel(logging.INFO)
        logging.getLogger('swagger_spec_validator.validator20').setLevel(logging.INFO)
        from logging.config import dictConfig
        dictConfig(logging_config)


# 测试配置（测试行情库）
config = ConfigBase()
# 生产配置
# config = ConfigProduct()


def update_config(config_update: ConfigBase):
    global config
    config = config_update
    logger.info('更新默认配置信息')
